/*******************************************************************************
 * This software is copyright (c) 2018 by
 * 
 *  - TextGrid Consortium (http://www.textgrid.de)
 *  - DAASI International GmbH (http://www.daasi.de)
 *
 * This is free software. You can redistribute it and/or modify it under the
 * terms described in the GNU Lesser General Public License v3 of which you
 * should have received a copy. Otherwise you can download it from
 *
 *   http://www.gnu.org/licenses/lgpl-3.0.txt
 *
 * @copyright TextGrid Consortium (http://www.textgrid.de)
 * @copyright DAASI International GmbH (http://www.daasi.de)
 * @license http://www.gnu.org/licenses/lgpl-3.0.txt GNU Lesser General Public
 * License v3
 * @author Stefan E. Funk (stefan.e.funk@daasi.de)
 ******************************************************************************/

package info.textgrid.middleware.tgauth.tgauthclient;

import static org.junit.Assert.assertTrue;

import java.io.FileInputStream;
import java.net.URL;
import java.util.Properties;

import javax.xml.ws.soap.SOAPFaultException;

import org.junit.AfterClass;
import org.junit.BeforeClass;
import org.junit.Test;

import info.textgrid.namespaces.middleware.tgauth_crud.AuthenticationFault;
import info.textgrid.namespaces.middleware.tgauth_crud.GetEPPNRequest;
import info.textgrid.namespaces.middleware.tgauth_crud.GetEPPNResponse;
import info.textgrid.namespaces.middleware.tgauth_crud.PortTgextraCrud;
import info.textgrid.namespaces.middleware.tgauth_crud.TgCrudCheckAccessRequest;
import info.textgrid.namespaces.middleware.tgauth_crud.TgCrudCheckAccessResponse;
import info.textgrid.namespaces.middleware.tgauth_crud.UnknownResourceFault;

/*******************************************************************************
 * TODOLOG
 * 
 * 	TODO	Add missing methods!
 * 
 *******************************************************************************
 * CHANGELOG
 * 
 *	2016-08-11	Funk	Using TGAuth client from TGAuth service utilities now!
 * 	2015-01-23	Funk	Combined WSDL's again :-)
 * 	2015-01-22	Funk	Separated the two WSDL's (Tgextra and Tgextra-crud).
 *
 ******************************************************************************/

/*******************************************************************************
 * <p>
 * This are TG-crud online tests for testing a running service to fulfil its
 * duties.
 * </p>
 * 
 * @author Stefan E. Funk, DAASI International GmbH
 * @version 2018-02-08
 * @since 2010-09-07
 ******************************************************************************/

public class TestTGAuthServiceOnline {

	// **
	// STATIC FINALS
	// **

	private static final String propertiesFile = "tgauth.test.devtextgridlaborg.crudjunit1.properties";

	// **
	// STATICS
	// **

	private static PortTgextraCrud	tgextraCrud;
	private static String			rbacSessionId;
	private static String			rbacSessionIdInvalid;
	private static String			crudSecret;
	private static String			crudSecretInvalid;
	private static String			logParameter;
	private static String			projectId;
	private static String			projectIdInvalid;
	private static String			username;
	private static String			usernameInvalid;
	private static String			usernameSearch;
	private static String			junitProject;

	/**
	 * @throws java.lang.Exception
	 */
	@BeforeClass
	public static void setUpBeforeClass() throws Exception {

		// Load properties file.
		Properties p = new Properties();
		p.load(new FileInputStream(.getResource(propertiesFile)));

		// Get and create the tgextra stubs.
		URL location = new URL(p.getProperty("tgextraServiceEndpointUrl"));
		URL locationCrud = new URL(
				p.getProperty("tgextraCrudServiceEndpointUrl"));

		System.out.println("WSDL location tgextra: " + location.getProtocol()
				+ "://" + location.getHost() + ":"
				+ (location.getPort() == -1 ? "80" : location.getPort())
				+ location.getPath());
		System.out.println("WSDL location tgextra-crud: "
				+ locationCrud.getProtocol() + "://" + locationCrud.getHost()
				+ ":"
				+ (locationCrud.getPort() == -1 ? "80" : locationCrud.getPort())
				+ locationCrud.getPath());

		initTgextraCrudService(locationCrud);

		// Get other needed things.
		rbacSessionId = p.getProperty("sessionId");
		rbacSessionIdInvalid = p.getProperty("sessionIdInvalid");
		junitProject = p.getProperty("junitProject");
		crudSecret = p.getProperty("crudSecret");
		crudSecretInvalid = p.getProperty("crudSecretInvalid");
		logParameter = p.getProperty("logParameter");
		projectIdInvalid = p.getProperty("projectIdInvalid");
		username = p.getProperty("username");
		usernameInvalid = p.getProperty("usernameInvalid");
		usernameSearch = p.getProperty("usernameSearch");

		// addProject();
	}

	/**
	 * @throws Exception
	 */
	@AfterClass
	public static void tearDownAfterClass() throws Exception {
		// deleteObjects();
		// deleteProject();
	}

	/**
	 * <p>
	 * Test the tgCheckAccess method with access granted on project#CREATE.
	 * </p>
	 * 
	 * @throws UnknownResourceFault
	 * @throws AuthenticationFault
	 */
	@Test
	public void testTgCrudCheckAccessProjectCreate()
			throws UnknownResourceFault, AuthenticationFault {

		System.out.println(
				"Testing #TGCRUDCHECKACCESS with access granted (project#CREATE)");

		TgCrudCheckAccessRequest req = new TgCrudCheckAccessRequest();
		req.setAuth(rbacSessionId);
		req.setSecret(crudSecret);
		req.setLog(logParameter);
		req.setOperation(OPERATION_CREATE);
		req.setResource(projectId);

		System.out.println("\tsession ID:    " + req.getAuth());
		System.out.println("\tcrud secret:   " + req.getSecret());
		System.out.println("\tlog parameter: " + req.getLog());
		System.out.println("\toperation:     " + req.getOperation());
		System.out.println("\tresource:      " + req.getResource());
		System.out.println(TABLINE);

		try {
			TgCrudCheckAccessResponse response = tgextraCrud
					.tgCrudCheckAccess(req);
			System.out.println("\tresult: " + response.isResult());

			System.out.println(TABLINE);
			System.out.println("\tOK");

		} catch (UnknownResourceFault e) {
			System.err.println("\tARGL! ERROR! " + e.getClass().getName() + ": "
					+ e.getMessage());
			throw e;
		} catch (AuthenticationFault e) {
			System.err.println("\tARGL! ERROR! " + e.getClass().getName() + ": "
					+ e.getMessage());
			throw e;
		}
	}

	/**
	 * <p>
	 * Test the tgCrudCheckAccess.
	 * </p>
	 * 
	 * @throws UnknownResourceFault
	 * @throws AuthenticationFault
	 */
	@Test
	public void testTgCrudCheckAccessGranted()
			throws UnknownResourceFault, AuthenticationFault {

		String operation = "create";

		System.out.println("Testing #TGCRUDCHECKACCESS with access granted");

		// Create checkAccess request.
		TgCrudCheckAccessRequest req = new TgCrudCheckAccessRequest();
		req.setAuth(rbacSessionId);
		req.setLog(logParameter);
		req.setOperation(operation);
		req.setResource(projectId);
		req.setSecret(crudSecret);

		System.out.println("\tsession ID:    " + req.getAuth());
		System.out.println("\tlog parameter: " + req.getLog());
		System.out.println("\toperation:     " + req.getOperation());
		System.out.println("\tresource:      " + req.getResource());
		System.out.println("\tsecret:        " + req.getSecret());
		System.out.println(TABLINE);

		try {
			TgCrudCheckAccessResponse response = tgextraCrud
					.tgCrudCheckAccess(req);

			System.out.println("\tusername:  " + response.getUsername());
			System.out.println("\toperation: " + response.getOperation());
			System.out.println("\tproject:   " + response.getProject().getId()
					+ " \"" + response.getProject().getName() + "\"");
			System.out.println("\tresult:    " + response.isResult());
			System.out.println("\tisPublic:  " + response.isPublic());

			System.out.println(TABLINE);
			System.out.println("\tOK");

		} catch (UnknownResourceFault e) {
			System.err.println("\tARGL! ERROR! " + e.getClass().getName() + ": "
					+ e.getMessage());
			throw e;
		} catch (AuthenticationFault e) {
			System.err.println("\tARGL! ERROR! " + e.getClass().getName() + ": "
					+ e.getMessage());
			throw e;
		}
	}

	/**
	 * <p>
	 * Test the tgCrudCheckAccess with invalid SID.
	 * </p>
	 * 
	 * @throws UnknownResourceFault
	 * @throws AuthenticationFault
	 */
	@Test
	public void testTgCrudCheckAccessDenied()
			throws UnknownResourceFault, AuthenticationFault {

		String operation = "create";

		System.out
				.println("Testing #TGCRUDCHECKACCESS with invalid session ID");

		// Create checkAccess request.
		TgCrudCheckAccessRequest req = new TgCrudCheckAccessRequest();
		req.setAuth(rbacSessionIdInvalid);
		req.setLog(logParameter);
		req.setOperation(operation);
		req.setResource(projectId);
		req.setSecret(crudSecret);

		System.out.println("\tsession ID:    " + req.getAuth());
		System.out.println("\tlog parameter: " + req.getLog());
		System.out.println("\toperation:     " + req.getOperation());
		System.out.println("\tresource:      " + req.getResource());
		System.out.println("\tsecret:        " + req.getSecret());
		System.out.println(TABLINE);

		try {
			TgCrudCheckAccessResponse response = tgextraCrud
					.tgCrudCheckAccess(req);

			System.out.println("\tresult: " + response.isResult());

			if (response.isResult()) {
				assertTrue(false);
			}

			System.out.println(TABLINE);
			System.out.println("\tOK");

		} catch (UnknownResourceFault e) {
			System.err.println("\tARGL! ERROR! " + e.getClass().getName() + ": "
					+ e.getMessage());
			throw e;
		} catch (AuthenticationFault e) {
			System.err.println("\tARGL! ERROR! " + e.getClass().getName() + ": "
					+ e.getMessage());
			throw e;
		}
	}

	/**
	 * <p>
	 * Test the tgCrudCheckAccess with invalid TG-crud secret.
	 * </p>
	 * 
	 * @throws AuthenticationFault
	 * @throws UnknownResourceFault
	 */
	@Test(expected = AuthenticationFault.class)
	public void testTgCrudCheckAccessNoSecret()
			throws AuthenticationFault, UnknownResourceFault {

		System.out.println(
				"Testing #TGCRUDCHECKACCESS with invalid tgcrud secret (project#CREATE)");

		// Create checkAccess request.
		TgCrudCheckAccessRequest req = new TgCrudCheckAccessRequest();
		req.setAuth(rbacSessionId);
		req.setLog(logParameter);
		req.setOperation(OPERATION_CREATE);
		req.setResource(projectId);
		req.setSecret(crudSecretInvalid);

		System.out.println("\tsession ID:    " + req.getAuth());
		System.out.println("\tlog parameter: " + req.getLog());
		System.out.println("\toperation:     " + req.getOperation());
		System.out.println("\tresource:      " + req.getResource());
		System.out.println("\tsecret:        " + req.getSecret());
		System.out.println(TABLINE);

		try {
			tgextraCrud.tgCrudCheckAccess(req);

		} catch (SOAPFaultException e) {
			if (e.getMessage().equals(
					"You are not cruddy enough - Go away and come back as CRUD!")) {

				System.out.println("\tOK");

				throw new AuthenticationFault(e.getMessage());
			}
		} catch (UnknownResourceFault e) {
			System.err.println("\tARGL! ERROR! " + e.getClass().getName() + ": "
					+ e.getMessage());
			throw e;
		}
	}

	/**
	 * <p>
	 * Test the tgCrudCheckAccess with invalid resource.
	 * </p>
	 * 
	 * @throws AuthenticationFault
	 * @throws UnknownResourceFault
	 * @throws AuthenticationFault
	 */
	@Test(expected = UnknownResourceFault.class)
	public void testTgCrudCheckAccessUnknownResource()
			throws UnknownResourceFault, AuthenticationFault {

		System.out.println(
				"Testing #TGCRUDCHECKACCESS with unknown project resource (project#CREATE)");

		// Create checkAccess request.
		TgCrudCheckAccessRequest req = new TgCrudCheckAccessRequest();
		req.setAuth(rbacSessionId);
		req.setLog(logParameter);
		req.setOperation(OPERATION_CREATE);
		req.setResource(projectIdInvalid);
		req.setSecret(crudSecret);

		System.out.println("\tsession ID:    " + req.getAuth());
		System.out.println("\tlog parameter: " + req.getLog());
		System.out.println("\toperation:     " + req.getOperation());
		System.out.println("\tresource:      " + req.getResource());
		System.out.println("\tsecret:        " + req.getSecret());
		System.out.println(TABLINE);

		try {
			tgextraCrud.tgCrudCheckAccess(req);

		} catch (SOAPFaultException e) {
			if (e.getMessage()
					.equals("The given resource is unknown to the system.")) {

				System.out.println("\tOK");

				throw new UnknownResourceFault(e.getMessage());
			}
		} catch (AuthenticationFault e) {
			System.err.println("\tARGL! ERROR! " + e.getClass().getName() + ": "
					+ e.getMessage());
			throw e;
		}
	}

	/**
	 * <p>
	 * Test getting the ePPn. No secret needed anymore!
	 * </p>
	 * 
	 * @throws AuthenticationFault
	 */
	@Test
	public void testTgCrudGetEppn() throws AuthenticationFault {

		System.out.println("Testing #TGCRUDGETEPPN");

		// Create getEPPN request.
		GetEPPNRequest req = new GetEPPNRequest();
		req.setAuth(rbacSessionId);
		req.setLog(logParameter);

		System.out.println("\tsession ID:    " + req.getAuth());
		System.out.println("\tlog parameter: " + req.getLog());
		System.out.println("\tsecret:        " + req.getSecret());
		System.out.println(TABLINE);

		try {
			GetEPPNResponse response = tgextraCrud.getEPPN(req);

			System.out.println("\teppn: " + response.getEppn());

			System.out.println(TABLINE);
			System.out.println("\tOK");

		} catch (AuthenticationFault e) {
			System.err.println("\tARGL! ERROR! " + e.getClass().getName() + ": "
					+ e.getMessage());
			throw e;
		}
	}

	/**
	 * <p>
	 * Initialises the TG-extra CRUD stub.
	 * </p>
	 * 
	 * @param theEndpoint
	 */
	private static void initTgextraCrudService(URL theEndpoint) {

		if (tgextraCrud == null) {

			// Create TG-extra service stubs.
			tgextraCrud = TGAuthClientCrudUtilities.getTgextraCrud(theEndpoint);

			System.out.println("TG-auth CRUD stub created");
		} else {
			System.out.println("Using existing TG-auth CRUD stub");
		}
	}

}
