/*******************************************************************************
 * This software is copyright (c) 2018 by
 *
 *   TextGrid Consortium (http://www.textgrid.de)
 *
 * This is free software. You can redistribute it and/or modify it under the
 * terms described in the GNU Lesser General Public License v3 of which you
 * should have received a copy. Otherwise you can download it from
 *
 *   http://www.gnu.org/licenses/lgpl-3.0.txt
 *
 * @copyright TextGrid Consortium (http://www.textgrid.de)
 * @copyright SUB Göttingen (http://www.sub.uni-goettingen.de)
 * @license http://www.gnu.org/licenses/lgpl-3.0.txt GNU Lesser General Public
 * License v3
 * @author Stefan E. Funk (funk@sub.uni-goettingen.de)
 ******************************************************************************/

package info.textgrid.middleware.tgauth.tgauthclient;

import static org.junit.Assert.assertTrue;

import java.net.MalformedURLException;
import java.net.URL;

import info.textgrid.namespaces.middleware.tgauth.BooleanResponse;
import info.textgrid.namespaces.middleware.tgauth.GetObjectsRequest;
import info.textgrid.namespaces.middleware.tgauth.PortTgextra;
import info.textgrid.namespaces.middleware.tgauth.ResourcesetResponse;
import info.textgrid.namespaces.middleware.tgauth.TgCheckAccessRequest;
import info.textgrid.namespaces.middleware.tgauth.UnknownResourceFault;

import org.junit.After;
import org.junit.AfterClass;
import org.junit.Before;
import org.junit.BeforeClass;
import org.junit.Ignore;
import org.junit.Test;

/*******************************************************************************
 * TODOLOG
 *
 *******************************************************************************
 * CHANGELOG
 *
 * 2016-08-05    Funk    Added getObjects test.
 * 2016-08-03    Funk    Copied from TGAuthClientUtilities.
 ******************************************************************************/

/*******************************************************************************
 * <p>
 * TG-auth client utility class to get appropriate TG-auth clients.
 * </p>
 * 
 * @author Stefan E. Funk, SUB Göttingen
 * @version 2018-03-06
 * @since 2016-08-04
 ******************************************************************************/

public final class TestTGAuthClientUtilities {

	/**
	 * @throws Exception
	 */
	@BeforeClass
	public static void setUpBeforeClass() throws Exception {
	}

	/**
	 * @throws java.lang.Exception
	 */
	@AfterClass
	public static void tearDownAfterClass() throws Exception {
	}

	/**
	 * @throws java.lang.Exception
	 */
	@Before
	public void setUp() throws Exception {
	}

	/**
	 * @throws java.lang.Exception
	 */
	@After
	public void tearDown() throws Exception {
	}

	/**
	 * <p>
	 * Note: This is an online test, please enable if necessary!
	 * </p>
	 * 
	 * @throws MalformedURLException
	 * @throws UnknownResourceFault
	 */
	@Ignore
	@Test(expected = UnknownResourceFault.class)
	public static void testGetTgextraCheckAccess()
			throws MalformedURLException, UnknownResourceFault {

		URL endpoint = new URL(
				"https://textgridlab.org/1.0/tgauth/tgextra.php");

		PortTgextra tgextra = TGAuthClientUtilities.getTgextra(endpoint);

		TgCheckAccessRequest request = new TgCheckAccessRequest();
		request.setAuth("urgl");
		request.setLog("");
		request.setOperation("read");
		request.setResource("textgrid:unknown");

		BooleanResponse response = tgextra.tgCheckAccess(request);

		System.out.println("checkAccess: " + response.isResult());
	}

	/**
	 * <p>
	 * Note: This is an online test, please enable if necessary!
	 * </p>
	 * 
	 * @throws MalformedURLException
	 * @throws UnknownResourceFault
	 */
	@Ignore
	@Test
	public static void testGetTgextraGetObjets()
			throws MalformedURLException, UnknownResourceFault {

		URL endpoint = new URL(
				"https://textgridlab.org/1.0/tgauth/tgextra.php");

		PortTgextra tgextra = TGAuthClientUtilities.getTgextra(endpoint);

		GetObjectsRequest request = new GetObjectsRequest();
		request.setAuth("");
		request.setLog("");
		// TG-rep, Veronica II
		request.setProject("TGPR-26236625-1acc-b921-a5fa-53567c3eeb80");
		ResourcesetResponse response = tgextra.getObjects(request);

		System.out.println("all objects: " + response.getResource());

		if (response.getResource().isEmpty()) {
			assertTrue(false);
		}
	}

}
